/*
  ==============================================================================
    jQuery LayerButton plugin               Copyright (C) TuneUp Software GmbH
  ==============================================================================
  
  USAGE EXAMPLE
  -------------

  $('.taskdlg-button').LayerButton(
    {
      'hover':
      {
        'imageSource':  { 'prefix': 'taskdlg_button_', 'suffix': '.png' },
        'imageBorders': { 'top': 3, 'right': 3, 'bottom': 3, 'left': 3 },
        'fadeIn':       { 'speed': 50,  'easing': 'easeOutQuad' },
        'fadeOut':      { 'speed': 150, 'easing': 'easeOutQuad' }
      }
    }
  );
*/

(function($)
{
  // Create our own namespace inside the jQuery namespace
  $.LayerButton = {};
  $.extend($.LayerButton, {
    // Internal unique IDs to identify groups and buttons
    runningGroupID:  0,
    runningButtonID: 0,
    // Automatically initializes some internal functionality on first use
    initialize: function()
    {
      if ($.LayerButton.initialized) return;
      $.LayerButton.initialized = true;
      window.setInterval( 
        function()
        {
          // Remove unused hidden layers after a while
          var curTime = (new Date()).getTime();
          $('.lb-layer').each(
            function()
            {
              if (this.hiddenSince)
              {
                if (curTime - this.hiddenSince >= 30000)
                {
                  $(this).remove();
                }
              }
            }
          );
        },
        1000
      );
    },
    getLayerID: function(aParent, aType, normalId)
    {
      if ((aType == 'hover') || (aType == 'normal'))
      {
        return 'lblayer' + aParent.LayerButton.uniqueGroupID + aType + (normalId ? normalId : '');
      }
      else
      {
        return 'lblayer' + aParent.LayerButton.uniqueButtonID + aType;
      }
    },
    // Creates a layer on the fly and inserts it into the DOM before aParent
    createLayer: function(aParent, aType, normalId)
    {
      $.LayerButton.initialize();
      var html_contents =
        '<table id="' + $.LayerButton.getLayerID(aParent, aType, normalId) + '" lbgroupid="' + aParent.LayerButton.uniqueGroupID + '" lbbuttonid="'
          + aParent.LayerButton.uniqueButtonID + '" class="lb-layer ' + aType
          + '" cellpadding="0" cellspacing="0" style="width: 10px; height: 10px; position: absolute; top: 0; left: 0; z-index: -1; table-layout: fixed; border-collapse: collapse;">';
      if ( ( aParent.LayerButton.amountTiles == 1 ) || ( aParent.LayerButton.amountTiles == '3h' ) ) {
        html_contents += '<col style="width: auto">';
      } else {
        html_contents +=
          '<col style="width: ' + aParent.LayerButton[ aType ].imageBorders.left + 'px"><col style="width: auto"><col style="width: '
            + aParent.LayerButton[ aType ].imageBorders.right + 'px">';
      }

      if ( ( aParent.LayerButton.amountTiles == 9 ) || ( aParent.LayerButton.amountTiles == '3h' ) ) {
        var padding = 'padding: 0px;';
      } else {
        var padding = 'padding: ' + aParent.LayerButton[ aType ].imageBorders.top + 'px 0 ' + aParent.LayerButton[ aType ].imageBorders.bottom + 'px 0;';

      }

      if ( aParent.LayerButton.amountTiles == 1 ) {
        html_contents += '<tr>' + '<<single>>' + '</tr>';
      } else if ( aParent.LayerButton.amountTiles == 9 ) {
        html_contents +=
          '<tr style="height: ' + aParent.LayerButton[ aType ].imageBorders.top + 'px;">' + '<<tl>><<tm>><<tr>>' + '</tr>' + '<tr style="height: 100%;">' + '<<ml>><<mm>><<mr>>'
            + '</tr>' + '<tr style="height: ' + aParent.LayerButton[ aType ].imageBorders.bottom + 'px;">' + '<<bl>><<bm>><<br>>' + '</tr>';
      } else if ( aParent.LayerButton.amountTiles == '3h' ) {
        html_contents +=
          '<tr style="height: ' + aParent.LayerButton[ aType ].imageBorders.top + 'px;">' + '<<tm>>' + '</tr>' + '<tr style="height: 100%;">' + '<<mm>>' + '</tr>'
            + '<tr style="height: ' + aParent.LayerButton[ aType ].imageBorders.bottom + 'px;">' + '<<bm>>' + '</tr>';
      } else {
        html_contents += '<tr>' + '<<ml>><<mm>><<mr>>' + '</tr>';
      }
      html_contents += '</table>';
      
      if (aParent.LayerButton[aType].imageSource)
      {
        html_contents =
          html_contents.replace( /<<(\w+)>>/g, '<td style="' + padding + ' background-image: url(\''
            + /* <<'#AbsoluteBaseURL#' + *//* >> */aParent.LayerButton[ aType ].imageSource.prefix + '$1' + aParent.LayerButton[ aType ].imageSource.suffix + '\');"></td>' );
      }
      else
      {
        html_contents = html_contents.replace( /<<(\w+)>>/g, '<td style="' + padding + '"></td>' );
      }
      var parent_element = $('.scrollbar-fix-inner');
      if (!parent_element.length)
      {
        parent_element = $('body');
      }
      if (aType == 'hover')
      {
        parent_element.append(html_contents);
      }
      else
      {
        parent_element = $(aParent).first();
        parent_element.after(html_contents);
      }
    },
    assignHoverLayer: function(aParent)
    {
      // Bind events
      $(aParent).hover(
        // Over
        function(event)
        {
          /*
           * NAB 15.06.2011: disable it if mouse button on another control was pressed
           */
          if (mouseDownPressed && !isMouseInRect(event)) return;
          
          if ($(aParent).hasClass('inactive') || $(aParent).hasClass('disabled')) return;
          
          // Only show the hover layer when no selected layer is visible
          var selectedLayer = $.LayerButton.findLayer(aParent, 'selected', false);
          if (selectedLayer.length && selectedLayer[0].visible) return;
          // Resize the hover element to fit the button size
          $.LayerButton.updateLayerDimensions(aParent, 'hover');
          // Fade in
          $.LayerButton.showLayer(aParent, 'hover');
        },
        // Out
        function()
        {
          // Fade out
          $.LayerButton.hideLayer(aParent, 'hover');
        }
      );
    },
    assignSelectedLayer: function(aParent)
    {
      // Bind events
      $(aParent).click(
        function(event, param)
        {
          // param will be passed as 4th argument in swapVisibleLayer to handle case of #settings->mousedown
          $(aParent).LayerButton_Select(param);
        }
      );
    },
    findLayer: function(aParent, aType, aAutoCreate, normalId)
    {
      // Try to find the layer
      var layerID = $.LayerButton.getLayerID(aParent, aType, normalId);
      var layer   = $('#' + layerID);
      // If the layer does not exist, create it on the fly
      if (!layer.length && aAutoCreate)
      {
        $.LayerButton.createLayer(aParent, aType, normalId);
        layer = $('#' + layerID);
      }
      return layer;
    },
    updateLayerDimensions: function(aParent, aType, normalId)
    {
      var layer = $.LayerButton.findLayer(aParent, aType, true, normalId);
      if (!layer.length) return;
      
      // Calculate new rectangle
      if (aType == 'hover')
      {
        var new_pos  = $(aParent).offset();
      }
      else
      {
        var new_pos  = $(aParent).position();
      }
      var new_size = {
        'width':      $(aParent).innerWidth(),
        'height':     $(aParent).innerHeight()
      };
      // Calculate current rectangle
      if (aType == 'hover')
      {
        var old_pos  = layer.offset();
      }
      else
      {
        var old_pos  = layer.position();
      }
      var old_size = {
        'width':      layer.innerWidth(),
        'height':     layer.innerHeight()
      };
      
      layer.find('tr:eq(1)').css(
        'height',
        (new_size.height - aParent.LayerButton[ aType ].imageBorders.bottom - aParent.LayerButton[ aType ].imageBorders.top) + 'px'
      );
      if ( new_pos.top > 1000 ) {
        layer.hide( );
      }
      // Update CSS properties of the hover element if necessary
      if (
           (new_pos.left    != old_pos.left) ||
           (new_pos.top     != old_pos.top) ||
           (new_size.width  != old_size.width) ||
           (new_size.height != old_size.height)
         )
      {
        // Store the previous opacity value
        var oldOpacity = layer.css('opacity');
        // Set the opacity to 0 to make the layer invisible, move it to its new
        // location and resize it to fit the parent LayerButton
        layer.css(
          {
            'opacity':    0,
            'left':       new_pos.left,
            'top':        new_pos.top,
            'width':      new_size.width,
            'height':     new_size.height
          }
        );
        // Restore the opacity back to its previous value
        layer.css(
          {
            'opacity':    oldOpacity
          }
        );
      }
    },
    resetLayer: function(aParent, aType)
    {
      var layer = $.LayerButton.findLayer(aParent, aType, false);
      if (!layer.length) return;

      layer.css(
        {
          'width':  '10px',
          'height': '10px'
        }
      );
    },
    showLayer: function(aParent, aType)
    {
      var layer = $.LayerButton.findLayer(aParent, aType, true);
      if (!layer.length) return;
      
      if (aParent.LayerButton[aType].onShow)
      {
        aParent.LayerButton[aType].onShow(aParent);
      }
      
      $.LayerButton._showLayer(
        layer
      );
    },
    hideLayer: function(aParent, aType)
    {
      var layer = $.LayerButton.findLayer(aParent, aType, false);
      if (!layer.length) return;
      
      if (aParent.LayerButton[aType].onHide)
      {
        aParent.LayerButton[aType].onHide(aParent);
      }
      
      $.LayerButton._hideLayer(
        layer
      );
    },
    _showLayer: function(aLayer)
    {
      aLayer[0].visible     = true;
      aLayer[0].hiddenSince = 0;
      
      aLayer.show();
    },
    _hideLayer: function(aLayer)
    {
      aLayer[0].visible = false;
      
      aLayer.hide();
      aLayer[0].hiddenSince = (new Date()).getTime();
    },
    swapVisibleLayer: function(aParent, aHideLayerType, aShowLayerType, skipOnShow)
    {
      var layerToHide = $.LayerButton.findLayer(aParent, aHideLayerType, false);
      var layerToShow = $.LayerButton.findLayer(aParent, aShowLayerType, true);
      if (layerToShow.length)
      {
        // NAB 08.06.2011: skip onshow for #settings->mousedown
        if (!skipOnShow && aParent.LayerButton[aShowLayerType].onShow)
        {
          aParent.LayerButton[aShowLayerType].onShow(aParent);
        }
        $.LayerButton._showLayer(
          layerToShow
        );
      }
      if (layerToHide.length)
      {
        if (aParent.LayerButton[aHideLayerType].onHide)
        {
          aParent.LayerButton[aHideLayerType].onHide(aParent);
        }
        $.LayerButton._hideLayer(
          layerToHide
        );
      }
    }
  });

  // Add new public functionality to all jQuery objects
  $.fn.extend(
    {
      LayerButton: function(aAttributes)
      {
        // Assign one unique group class ID to the whole group of
        // selected elements
        var uniqueGroupID = '';
        if (aAttributes.groupID)
        {
          uniqueGroupID = aAttributes.groupID;
        }
        else
        {
          $.LayerButton.runningGroupID = $.LayerButton.runningGroupID + 1;
          uniqueGroupID = $.LayerButton.runningGroupID;
        }
        // Initialize LayerButton functionality for each of the
        // selected elements one by one
        return this.each(
          function()
          {
            // Save the given LayerButton attributes in the DOM node
            // so that they can be accessed later
            this.LayerButton = {};
            $.extend(this.LayerButton, aAttributes);
            // Create a unique button class ID
            $.LayerButton.runningButtonID = $.LayerButton.runningButtonID + 1;
            $.extend(this.LayerButton, {
              'uniqueGroupID':  uniqueGroupID,
              'uniqueButtonID': $.LayerButton.runningButtonID
            });
            $(this)
              .addClass('lb-button')
              .attr('lbgroupid',  this.LayerButton.uniqueGroupID)
              .attr('lbbuttonid', this.LayerButton.uniqueButtonID);
            
            if (this.LayerButton.normal)
            {
              $.LayerButton.findLayer(this, 'normal', true, this.id);
              $.LayerButton.updateLayerDimensions(this, 'normal', this.id);
            }
            
            // Has a hover layer been specified?
            if (this.LayerButton.hover)
            {
              // Yes, bind events
              $.LayerButton.assignHoverLayer(this);
            }
            // Has a selected layer been specified?
            if (this.LayerButton.selected)
            {
              // Yes, bind events
              $.LayerButton.assignSelectedLayer(this);
            }
            // Do we need to listen to events?
            if (this.LayerButton.events)
            {
              // Handle click event
              if (this.LayerButton.events.click)
              {
                $(this).bind('click', this.LayerButton.events.click);
              }
            }
          }
        );
      },
      LayerButton_Select: function(param)
      {
        return this.each(
          function()
          {
            var clickedButtonID = $(this).attr('lbbuttonid');
            if (!clickedButtonID) return;
    
            // Resize the selected element to fit the button size
            $.LayerButton.updateLayerDimensions(this, 'selected');
    
            // Hide hover layer and show selected layer at the same time
            $.LayerButton.swapVisibleLayer(this, 'hover', 'selected', param);
    
            // Check if the clicked button belongs to a group
            var clickedGroupID = $(this).attr('lbgroupid');
            if (clickedGroupID)
            {
              // Find previously selected button(s) within the same group
              var previouslySelected = $('.lb-button[lbgroupid=' + clickedGroupID + '][lbbuttonid!=' + clickedButtonID + ']');
              // Hide selected layer of previously selected button (if any)
              if (previouslySelected.length)
              {
                previouslySelected.each(
                  function()
                  {
                    $.LayerButton.hideLayer(this, 'selected');
                  }
                );
              }
            }
          }
        );
      }
    }
  );
})(jQuery);
